@@ -1287,7 +1287,8 @@ gok_spy_distinguish_node_name (AccessibleNode *node)
 						AccessibleRelation_getTarget(relations[i], j);
 					if (labelAccessible != NULL) {
 						s = Accessible_getName (labelAccessible);
-						if (s != NULL) {
+						/* don't prepend if we've already used this label as GokButton name */
+						if (s != NULL && strcmp (s, node->pname)) {
 							if ( strlen (s) != 0 )  {
 								gchar buffer[64] = "";
 								if (node->pname) 
@@ -2323,8 +2324,6 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 		}
 	}	
 
-	gok_spy_resolve_namesakes (_priv_ui_nodes);
-
 	/* one last check for context menus on the currently focussed object */
 	if (keyboard_ui_flags.data.menus)
 	{
@@ -2358,6 +2357,8 @@ gok_spy_update_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 	gok_spy_free_nodes (_priv_ui_nodes);
 	_priv_ui_nodes = NULL;
 	ui_flags = gok_spy_search_component_list (rootAccessible, keyboard_ui_flags);
+	/* _priv_ui_nodes is now re-assigned, by above call. */
+	gok_spy_resolve_namesakes (_priv_ui_nodes);
 
 	gok_log_leave ();
 
