@@ -87,6 +87,8 @@
 
 static gboolean respawn_on_segv = TRUE;
 
+static GokApplication *_gok_app = NULL;
+
 /* pointer to the first keyboard in the list of keyboards */
 static GokKeyboard* m_pKeyboardFirst = NULL;
 
@@ -356,10 +358,18 @@ gok_main_get_custom_compose_kbd_name ()
 	return gok_args.custom_compose_kbd_name;
 }
 
+gboolean
+gok_main_safe_mode ()
+{
+	return _gok_app->safe;
+}
+
 static gboolean
 gok_application_set_safe (LoginHelper *helper, gboolean safe)
 {
-    return TRUE; /* TODO: actually go to safe mode here! */
+    GOK_APPLICATION (helper)->safe = safe;
+    gok_main_display_scan_reset ();
+    return TRUE;
 }
 
 static LoginHelperDeviceReqFlags
@@ -391,7 +401,7 @@ gok_application_get_raise_windows (LoginHelper *helper)
 static void
 gok_application_init (GokApplication *app)
 {
-    ;
+    app->safe = FALSE;
 }
 
 static void
@@ -518,7 +528,8 @@ gok_main_open(gint argc, gchar *argv[])
 
 	    CORBA_exception_init (&ev);
 
-	    obj = BONOBO_OBJECT (g_object_new (GOK_TYPE_APPLICATION, NULL));
+	    _gok_app = g_object_new (GOK_TYPE_APPLICATION, NULL);
+	    obj = BONOBO_OBJECT (_gok_app);
 	    display_name = gdk_display_get_name (gdk_display_get_default ());
 	    reg_env = bonobo_activation_registration_env_set (reg_env, "DISPLAY",
 							      display_name);
@@ -2592,7 +2603,7 @@ gok_main_display (void)
 {
 	if (gok_main_get_main_window () != NULL)
 		return GDK_WINDOW_XDISPLAY (gok_main_get_main_window ()->window);
-	return NULL;
+	return GDK_DISPLAY ();
 }
 
 
@@ -2724,9 +2735,9 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 			xkb_ok = TRUE;
 		} 
 		if (!xkb || !(xkb->ctrls->enabled_ctrls & XkbStickyKeysMask)) {
+			gboolean sticky_is_set = FALSE;
 			/* Turn on sticky keys and warn user! */
-#if USE_GCONF
-			if (1) {
+			if (!gok_args.is_login) {
 				gboolean keyboard_access;
 				GConfClient *gconf_client = gconf_client_get_default ();
 				gok_gconf_get_bool (gconf_client, KEYBOARD_ACCESSIBILITY_ENABLE_KEY, &keyboard_access);
@@ -2734,37 +2745,42 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 					gok_gconf_set_bool (gconf_client, KEYBOARD_ACCESSIBILITY_ENABLE_KEY, TRUE); /* set keyboard accessibility key before setting sticky keys */
 				gok_gconf_set_bool (gconf_client, KEYBOARD_ACCESSIBILITY_STICKY_KEYS_KEY, 
 						    TRUE); /* turn on gconf key */
-#else
-			if (XkbChangeEnabledControls (display,
+				xkb_ok = TRUE;
+				sticky_is_set = TRUE;
+			}
+			else if (XkbChangeEnabledControls (display,
 						      XkbUseCoreKbd,
 						      XkbStickyKeysMask,
 						      XkbStickyKeysMask)) {
-#endif
 				xkb_ok = TRUE;
-				if (!gok_args.is_login) {
-				  dialog = gtk_message_dialog_new (NULL,
+				sticky_is_set = TRUE;
+				g_message ("Sticky keys enabled.");
+			}
+			else
+			{
+				g_warning (_("GOK may not work properly, because it could not enable your desktop's 'sticky keys' feature."));
+			}
+			if (!gok_args.is_login && sticky_is_set) {
+				dialog = gtk_message_dialog_new (NULL,
 								 0,
 								 GTK_MESSAGE_INFO,
 								 GTK_BUTTONS_OK,
 								 _("GOK has enabled Sticky Keys, which it requires.\n"));
 				
-				  /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
-				  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
+				/* Destroy the dialog when the user responds to it (e.g. clicks a button) */
+				g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
 							  G_CALLBACK (gtk_widget_destroy),
 							  GTK_OBJECT (dialog));
-
-				  /* reset hint to avoid self-occlusion */
-				  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
-				  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-				  gtk_widget_show_all (dialog);
-				}
+				
+				/* reset hint to avoid self-occlusion */
+				gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
+				gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+				gtk_widget_show_all (dialog);
 			}
-			else
-				xkb_ok = FALSE;
 		}
 		XkbFreeKeyboard (xkb, 0, True);
 	}
-	if (xkb_ok == FALSE) {
+	else {
 		if (!gok_args.is_login) {
                 /* post an error dialog */
 			dialog = gtk_message_dialog_new (NULL,
@@ -2777,6 +2793,8 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 					  GTK_OBJECT (dialog));
 			gtk_widget_show_all (dialog);
 		}
+		else
+			g_warning ("XKB extension not present or non-functional.  GOK will exit.");
 		return FALSE;
 	}
 	return TRUE;
