@@ -43,6 +43,7 @@ typedef struct _GokApplication GokApplication;
 
 struct _GokApplication {
   LoginHelper     parent;
+  gboolean        safe;
 };
 
 typedef struct _GokApplicationClass {
@@ -50,6 +51,8 @@ typedef struct _GokApplicationClass {
 } GokApplicationClass;
 
 /* public */
+GType      gok_application_get_type (void) G_GNUC_CONST;
+
 GtkWidget* gok_main_create_window (gboolean is_dock);
 void gok_main_set_cursor (GdkCursor *cursor);
 gint gok_main_open(gint argc, gchar *argv[]);
@@ -86,6 +89,7 @@ gboolean gok_main_get_login (void);
 gchar* gok_main_get_inputdevice_name (void);
 void gok_main_check_accessibility (void);
 void gok_main_warn_if_corepointer_mode (gchar *message, gboolean always);
+gboolean gok_main_safe_mode (void);
 
 /* private */
 void gok_main_app_change_listener (Accessible* pAccessible);
